@@ -31,8 +31,20 @@ class PPLine(GridElement):
         self._check(time)
         self.set_value("in_service", self.in_service)
 
-        if not self.in_service:
-            LOG.debug(f"At step {time}: Line {self.index} out of service.")
-            pp.runpp(self.grid)
+        if old_state != self.in_service:
+            if not self.in_service:
+                LOG.debug(f"At step {time}: Line {self.index} out of service.")
+                try:
+                    pp.runpp(self.grid)
+                except pp.LoadflowNotConverged:
+                    LOG.debug("Line disabled. PF still not converging.")
+            else:
+                LOG.debug(
+                    f"At step {time}: Line {self.index} back in service."
+                )
+                try:
+                    pp.runpp(self.grid)
+                except pp.LoadflowNotConverged:
+                    LOG.debug("Line re-enabled. PF not converging.")
 
         return old_state != self.in_service
