@@ -217,6 +217,7 @@ class PowergridModule(UpgradeModule):
                     }
                 )
             if entity.type in ("Load", "Sgen"):
+                q_min = 0
                 try:
                     pp_grid = list(
                         self.scenario.get_sim(
@@ -226,11 +227,16 @@ class PowergridModule(UpgradeModule):
                     _, etype, eidx, _ = entity.eid.split("-")
                     p_max = pp_grid[etype].p_mw.loc[int(eidx)]
                     q_max = pp_grid[etype].q_mvar.loc[int(eidx)]
+
                 except Exception as err:
                     print("In PowergridModule: Caught exception:", err)
                     p_max = 0.5
                     q_max = 0.5
 
+                if q_max < q_min:
+                    q_min = q_max
+                    q_max = 0
+
                 self.scenario.actuators.append(
                     {
                         "actuator_id": f"{entity.sid}.{entity.eid}.p_mw",
@@ -244,7 +250,7 @@ class PowergridModule(UpgradeModule):
                     {
                         "actuator_id": f"{entity.sid}.{entity.eid}.q_mvar",
                         "action_space": (
-                            f"Box(low=0, high={q_max}, shape=(1,), dtype="
+                            f"Box(low={q_min}, high={q_max}, shape=(1,), dtype="
                             "np.float32)"
                         ),
                     }
