@@ -2,7 +2,10 @@
 pandapower model.
 
 """
+import json
 import logging
+import sys
+from typing import Any, Dict, List
 
 import numpy as np
 import mosaik_api
@@ -19,6 +22,7 @@ from .plotter import Plotter
 # Need to hard code the name because if started as separate
 # procress, the __name__ is __main__ instead.
 LOG = logging.getLogger("midas.modules.powergrid.simulator")
+# KVLOG = logging.getLogger("midas.modules.powergrid.simulator.keyvalue")
 
 
 class PandapowerSimulator(mosaik_api.Simulator):
@@ -26,12 +30,12 @@ class PandapowerSimulator(mosaik_api.Simulator):
 
     def __init__(self):
         super().__init__(META)
-        self.sid = None
-        self.models = dict()
-        self.entity_map = dict()
-        self.entities = list()
+        self.sid: str = ""
+        self.models: Dict[str, PandapowerGrid] = {}
+        self.entity_map: dict = {}
+        self.entities: list = []
 
-        self.step_size = None
+        self.step_size: int = 0
         self.now_dt = None
         self.cache = dict()
 
@@ -176,7 +180,7 @@ class PandapowerSimulator(mosaik_api.Simulator):
         """
         self._sim_time = time
         if self.key_value_logs:
-            LOG.info(inputs)
+            LOG.info(json.dumps(inputs))
         else:
             LOG.debug("At step %d received inputs %s", time, mformat(inputs))
 
@@ -237,7 +241,9 @@ class PandapowerSimulator(mosaik_api.Simulator):
 
         return time + self.step_size
 
-    def get_data(self, outputs):
+    def get_data(
+        self, outputs: Dict[str, List[str]]
+    ) -> Dict[str, Dict[str, Any]]:
         """Return the requested outputs (if feasible).
 
         Parameters
@@ -252,51 +258,75 @@ class PandapowerSimulator(mosaik_api.Simulator):
 
         """
 
-        data = dict()
-        grid_json = dict()
+        data: dict = {}
+        grid_json: dict = {}
+
         for eid, attrs in outputs.items():
-
             if "Grid" in eid:
-                if "health" in attrs:
-                    health = (
-                        self.models[eid].grid.res_bus.vm_pu.values[1:].mean()
-                    )
-                    if np.isnan(health):
-                        LOG.debug(
-                            "Grid health is NaN. This might be okay so "
-                            "setting health to 0. Just that you know!"
-                        )
-                        health = 0
-                    data.setdefault(eid, dict())["health"] = health
-                if "grid_json" in attrs:
-                    grid_json.setdefault(eid, dict())[
-                        "grid_json"
-                    ] = self.models[eid].to_json()
-
-                continue
-            log_msg = {
-                "id": f"{self.sid}_{eid}",
-                "name": eid,
-                "type": eid.split("-")[1],
-            }
-            for attr in attrs:
-                val = self.cache[eid][attr]
-                if np.isnan(val):
-                    LOG.debug(f"{attr} for {eid} is NaN. Setting to 0.")
-                    val = 0
-                data.setdefault(eid, dict())[attr] = val
-                log_msg[attr] = val
-
-            if self.key_value_logs:
-                LOG.info(log_msg)
+                self._get_grid_attributes(eid, attrs, data, grid_json)
+            else:
+                self._get_children_attributes(eid, attrs, data)
 
         if not self.key_value_logs:
             LOG.debug(
                 "At step %d gathered outputs %s", self._sim_time, mformat(data)
             )
+
         update(data, grid_json)
         return data
 
+    def _get_grid_attributes(
+        self,
+        eid: str,
+        attrs: List[str],
+        data: Dict[str, Dict[str, Any]],
+        grid_json: Dict[str, Any],
+    ) -> None:
+
+        if "health" in attrs:
+            health = self.models[eid].grid.res_bus.vm_pu.values[1:].mean()
+            if np.isnan(health):
+                LOG.debug(
+                    "Grid health is NaN. This might be okay so "
+                    "setting health to 0. Just that you know!"
+                )
+                health = 0
+            data.setdefault(eid, dict())["health"] = health
+
+        if "grid_json" in attrs:
+            grid_json.setdefault(eid, dict())["grid_json"] = self.models[
+                eid
+            ].to_json()
+
+    def _get_children_attributes(
+        self,
+        eid: str,
+        attrs: List[str],
+        data: Dict[str, Dict[str, Any]],
+    ) -> None:
+
+        log_msg = {
+            "id": f"{self.sid}_{eid}",
+            "name": eid,
+            "type": eid.split("-")[1],
+        }
+        for attr in attrs:
+            val = self.cache[eid][attr]
+            if np.isnan(val):
+                LOG.debug(f"{attr} for {eid} is NaN. Setting to 0.")
+                val = 0
+            data.setdefault(eid, dict())[attr] = val
+            if isinstance(val, np.bool_):
+                val = bool(val)
+            log_msg[attr] = val
+
+        if self.key_value_logs:
+            try:
+                LOG.info(json.dumps(log_msg))
+            except TypeError:
+                print(log_msg)
+                sys.exit(-1)
+
 
 if __name__ == "__main__":
     set_and_init_logger(
